/**
 * Диалог, показывающий процесс загрузки чего либо.
 *
 * @param title String заголовок диалога
 * @param width Integer ширина диалога. По умолчанию 300.
 * @param height Integer высота диалога. По умолчанию auto.
 * @param position Object расположение диалога. Значение должен быть объектом с двумя полями - {top: newTop, left: newLeft}. По умолчанию: {"left": "center", "top": "center"}.
 */

function JProgressBar(data)
{
	data = data? data : {};

	var obj = {};
	obj._name = "";

	obj._width = "width" in data? data.width : 300;
	obj._height = "height" in data? data.height : "auto";
	obj._title = "title" in data? data.title : "Обработка запроса";
	obj._position = "position" in data? data.position : {"left": "center", "top": "center"};
	obj._src = null;

	/**
	 * Максимальное значение шкалы прогресса
	 * @type Integer
	 */
	obj._count = "count" in data? data.count : 0;

	/**
	 * Текущее значение шкалы прогресса
	 * @type Integer
	 */
	obj._current = "current" in data? data.current : 0;

	/**
	 * Ссылка на HTML-объект диалога
	 * @type jQuery
	 */
	obj._element = null;

	/**
	 * Ссылка на объект, содержащий фон. Фон перекрывает все элементы на странице.
	 * @type Splash
	 */
	obj._splash = null;

	/**
	 * Массив установленых таймеров
	 * @type Array
	 */
	obj._timeoutId = [];

	/**
	 * Время начала показа диалога. Диалог появляется через _TIMEOUT мс.
	 * @type Date
	 */
	obj._time = null;

    /**
     * Возвращает имя объекта
     */
    obj.getName = function()
    {
        return obj._name;
    }

	/**
	 * Добавляет идентификатор запущенного таймера в массив
	 * @param timeoutId Integer идентификатор установленного таймера
	 */
	obj.setTimeoutId = function(timeoutId)
	{
		obj._timeoutId.push(timeoutId);
	}

	/**
	 * Устанавливает значение ширины диалога
	 * @param width Integer
	 */
	obj.setWidth = function(width)
	{
		obj._width = width;
	}

	/**
	 * Устанавливает значение высоты диалога
	 * @param height Integer
	 */
	obj.setHeight = function(height)
	{
		obj._height = height;
	}

	/**
	 * Устанавливает значение заголовка диалога
	 * @param title String
	 */
	obj.setTitle = function(title)
	{
		obj._title = title;
	}

	/**
	 * Устанавливает значение позиции диалога.
	 * Значение должен быть объектом с двумя полями - {top: newTop, left: newLeft}
	 * @param position Object
	 */
	obj.setPosition = function(position)
	{
		obj._position = position;
	}

	/**
	 * Устанавливает максимальное значение шкалы прогресса диалога
	 * @param count Integer
	 */
	obj.setCount = function(count)
	{
		obj._count = count;
	}

	/**
	 * Первоначальная инициализация диалога.
	 * Создаётся HTML-объект и устанавливается связка с фоном.
	 * Фон перекрывает все элементы на странице.
	 */
	obj._init = function(initData)
	{
		//Нужна предзагрузка картинки
		if("preload" in initData)
		{
			obj._src = initData.preload;
		}
		obj._name = JProgressBar._static.prefix + JProgressBar._static.count++;
		var classNames = "class" in initData? JProgressBar._static.prefix + " " + initData["class"] : JProgressBar._static.prefix;
		obj._element = jQuery("<div />").attr({"id": obj._name, "class": classNames}).css("display", "none");
		if(Splash)
		{
			obj._splash = new Splash(initData);
		}
	}

	/**
	 * Вычиляет отступ окна диалога от левого края
	 * @return String
	 */
	obj._getLeft = function()
	{
		var left = obj._position.left;
		var width = obj._width;
		if(typeof left == "string" && left.toLowerCase() == "center")
		{
			left = Math.round((obj._element.parent().width() - width) / 2);
		}
		return left;
	}

	/**
	 * Вычиляет отступ окна диалога от верхнего края
	 * @return String
	 */
	obj._getTop = function()
	{
		var element = obj._element;
		var top = obj._position.top;
		var height = obj._height;
		if(typeof top == "string" && top.toLowerCase() == "center")
		{
			if(typeof height == "string" && height.toLowerCase() == "auto") height = element.height();
			top = Math.round((element.parent().height() - height) / 2);
		}
		return top;
	}

	/**
	 * Возвращает объект, который содержит вычисленные параметры размера и положения окна диалога.
	 * Объект содержит: {"width": newWidth, "height": newHeight, "left": newLeft, "top": newTop}
	 * @return Object
	 */
	obj._getCSS = function()
	{
		return {"width": obj._width, "height": obj._height, "left": obj._getLeft(), "top": obj._getTop()};
	}

	/**
	 * Возвращает HTML-вёрстку окна диалога.
	 * Если максимальное значение шкалы прогресса (count) не задано или равно 0, то шкала не будет показана
	 * @return String
	 */
	obj._getHTML = function()
	{
		return '<div class="progressBar">' +
			'<div class="title">' + obj._title + '</div>' +
			(obj._src? '<div class="progressImg"><img src="' + obj._src + '" /></div>' : "") +
			(obj._count && obj._count > 0? '<div class="progressBar"><div class="progressLine" id="' + obj.getName() + 'progressLine" style="width: 0%;" /><div id="' + obj.getName() + 'progressCounter" class="progressCounter">(0 из ' + obj._count + ')</div></div>' : '')
			/*+'<div class="controls"><button id="' + obj.getName() + 'pbClose" onclick="' + obj.getName() + '._destroy();" type="button">Закрыть</button></div></div>'*/;
	}

	/**
	 * Сбрасывает параметры диалога:
	 * - Загорловок
	 * - Максимальное значение шкалы прогресса
	 * - Текущее значение прогресса
	 * - Время начала показа окна диалога
	 * - Установленные таймеры
	 *
	 * @return JProgressBar
	 */
	obj.clear = function()
	{
		obj._title = "";
		obj._count = 0;
		obj._current = 0;
		obj._time = null;
		for(var i = 0, l = obj._timeoutId.length; i < l; i++)
		{
			clearTimeout(obj._timeoutId[i]);
		}
		obj._timeoutId = [];
	}

	/**
	 * Показывает окно диалога и фон, а также устанавливает время начала показа окна диалога
	 */
	obj.show = function()
	{
		obj._time = new Date();
		if(obj._splash) obj._splash.show();
		if(obj._element) obj._element.css({"z-index": ++JCore._static.zIndex}).show();
		return obj;
	}

	/**
	 * Скрывает окно диалога и фон, а также сбрасывает время начала показа окна диалога
	 * @return JProgressBar
	 */
	obj.hide = function()
	{
		if(obj._element) obj._element.hide();
		if(obj._splash) obj._splash.hide();
		obj._time = null;
		return obj;
	}

	/**
	 * Добавляет окно диалога в parent.
	 * Если parent не указан, то диалог добавляется в body.
	 * @param parent jQuery объект, в который диалог будет добавлен
	 * @return JProgressBar
	 */
	obj.append = function(parent)
	{
		if(!parent) parent = jQuery("body");
		if(obj._splash && obj._element && parent)
		{
			obj._splash.append(parent);
			obj._element.appendTo(parent);
			obj._element.html(obj._getHTML()).css(obj._getCSS());
			obj.setTimeoutId(setTimeout(obj.getName() + '.show();', JProgressBar._static._TIMEOUT_SHOW));
		}
		return obj;
	}

	/**
	 * Управляет индикатором прогресса
	 */
	obj.progress = function()
	{
		obj._current++;
		var progressLine = obj._element.find("#" + obj.getName() + "progressLine");
		var progressCounter = obj._element.find("#" + obj.getName() + "progressCounter");
		if(progressLine && progressCounter)
		{
			progressLine.css("width", (obj._current * 100/obj._count) + "%");
			progressCounter.html('(' + obj._current + ' из ' + obj._count + ')');
		}
		progressCounter = progressLine = null;
	}

	/**
	 * Удаляет диалог. Предварительно его скрывает, а затем очищает.
	 */
	obj._destroy = function()
	{
		obj.hide();
		obj.clear();
		/* удаляем с возможностью восстановления */
		if(obj._element) obj._element.detach();
		if(obj._splash) obj._splash.destroy();
		obj._element = null;
		obj._splash = null;
		//console.log("destroy: " + obj.getName());
	}

	/**
	 * Удаляет диалог с задержкой _TIMEOUT мс.
	 * Диалог будет показан минимум _TIMEOUT мс, а после этого унечтожен.
	 */
	obj.destroy = function()
	{
		var time = new Date() - obj._time;
		if(time < JProgressBar._static._TIMEOUT_HIDE)
		{
			obj.setTimeoutId(setTimeout(function(){obj.destroy();}, JProgressBar._static._TIMEOUT_HIDE - time));
			return;
		}
		obj._destroy();
	}

	obj._init(data);
	window[obj.getName()] = obj;
	return obj;
}

/**
 * Статичные методы и параметры
 */
JProgressBar._static = {};
JProgressBar._static.prefix = "JProgressBar";
JProgressBar._static.count = 0;

/**
 * Время через которое будет показан индикатор процесса загрузки, обработки.
 * Значение задётся в милисекундах
 */
JProgressBar._static._TIMEOUT_SHOW = 2000;

/**
 * Минимальное время показа индикатор процесса загрузки, обработки.
 * Значение задётся в милисекундах
 */
JProgressBar._static._TIMEOUT_HIDE = 2000;

/**
 * Уничтожает все объекты JProgressBar
 */
JProgressBar._static.destroy = function()
{
	//console.log("JProgressBar:")
	for(var i = 0; i < JProgressBar._static.count; i++)
	{
		var name = JProgressBar._static.prefix + i;
		var el = window[name];
		if(!el) continue;
		el._destroy();
		window[name] = null;
	}
};